package cz.fi.muni.pa165.config;

import cz.fi.muni.pa165.dao.*;
import cz.fi.muni.pa165.entity.*;
import cz.fi.muni.pa165.enums.BandType;
import cz.fi.muni.pa165.enums.MusicianType;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


@Component
public class DataInitialization {
    private final ActivityDao activityDao;
    private final AlbumDao albumDao;
    private final BandDao bandDao;
    private final ManagerDao managerDao;
    private final MusicianDao musicianDao;
    private final OfferDao offerDao;
    private final SongDao songDao;

    public DataInitialization(ActivityDao activityDao,
                              AlbumDao albumDao,
                              BandDao bandDao,
                              ManagerDao managerDao,
                              MusicianDao musicianDao,
                              OfferDao offerDao,
                              SongDao songDao){
        this.activityDao = activityDao;
        this.albumDao = albumDao;
        this.bandDao = bandDao;
        this.managerDao = managerDao;
        this.musicianDao = musicianDao;
        this.offerDao = offerDao;
        this.songDao = songDao;
    }

@Transactional
    public void createMusicians(){
        Musician musicianEric = new Musician();
        musicianEric.setId("0");
        musicianEric.setType(MusicianType.GUITARIST);
        musicianEric.setName("Eric Stone");
        musicianDao.create(musicianEric);

        Musician musicianLea = new Musician();
        musicianLea.setName("Lea Lee");
        musicianLea.setType(MusicianType.SINGER);
        musicianLea.setId("1");
        musicianDao.create(musicianLea);

        Musician musicianTadeas = new Musician();
        musicianTadeas.setName("Tadeas Smith");
        musicianTadeas.setType(MusicianType.DRUMMER);
        musicianTadeas.setId("2");
        musicianDao.create(musicianTadeas);

        Musician musicianRebecca = new Musician();
        musicianRebecca.setName("Rebecca Fischer");
        musicianRebecca.setType(MusicianType.PIANIST);
        musicianRebecca.setId("3");
        musicianDao.create(musicianRebecca);

        Musician musicianMatt = new Musician();
        musicianMatt.setName("Matthew Tuck");
        musicianMatt.setType(MusicianType.SINGER);
        musicianMatt.setId("4");
        musicianDao.create(musicianMatt);

        Musician musicianMichael = new Musician();
        musicianMichael.setName("Michael Paget");
        musicianMichael.setType(MusicianType.GUITARIST);
        musicianMichael.setId("5");
        musicianDao.create(musicianMichael);

        Musician musicianJamie = new Musician();
        musicianJamie.setName("Jamie Mathias");
        musicianJamie.setType(MusicianType.BASS_GUITARIST);
        musicianJamie.setId("6");
        musicianDao.create(musicianJamie);

        Musician musicianJason = new Musician();
        musicianJason.setName("Jason Bowld");
        musicianJason.setType(MusicianType.DRUMMER);
        musicianJason.setId("7");
        musicianDao.create(musicianJason);
    }

    @Transactional
    public void createBand(){
        Band bandBFMV  = new Band();
        bandBFMV.setId("0");
        bandBFMV.setName("Bullet For My Valentine");
        bandBFMV.setType(BandType.METAL);
        bandBFMV.setLogo("BFMV");

        Musician matt = musicianDao.findById("4");
        if (matt == null){
         //   log.warn("Musician Matthew Tuck does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(matt);

        Musician michael = musicianDao.findById("5");
        if (michael == null){
        //    log.warn("Musician Michael Paget does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(michael);

        Musician jamie = musicianDao.findById("6");
        if (jamie == null){
        //    log.warn("Musician Jamie Mathias does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(jamie);

        Musician jason = musicianDao.findById("7");
        if (jason == null){
          //  log.warn("Musician Jason Bowld does not exists!");
            throw new AssertionError();
        }
        bandBFMV.addMusician(jason);

        bandDao.create(bandBFMV);

        Band bottleOfWatter = new Band();
        bottleOfWatter.setName("Bottle of Watter");
        bottleOfWatter.setType(BandType.PUNK);
        bottleOfWatter.setId("1");
        bandDao.create(bottleOfWatter);
    }

    @Transactional
    public void createSongs(){
        Song tears = new Song();
        tears.setSongName("Tears Don't Fall");
        tears.setDuration(Duration.ofMinutes(6));
        tears.setId("0");
        songDao.create(tears);

        Song betrayal = new Song();
        betrayal.setSongName("Your Betrayal");
        betrayal.setDuration(Duration.ofMinutes(5));
        betrayal.setId("1");
        songDao.create(betrayal);

        Song letting = new Song();
        letting.setSongName("Letting You Go");
        letting.setDuration(Duration.ofMinutes(4));
        letting.setId("2");
        songDao.create(letting);

        Song hearts = new Song();
        hearts.setSongName("Hearts Burst into Fire");
        hearts.setDuration(Duration.ofMinutes(5));
        hearts.setId("3");
        songDao.create(hearts);

        Song breathe = new Song();
        breathe.setSongName("Breathe Underwater");
        breathe.setDuration(Duration.ofMinutes(4));
        breathe.setId("4");
        songDao.create(breathe);
    }

    @Transactional
    public void createAlbum(){
        Album bestOf = new Album();
        Song tears = songDao.findById("0");
        if (tears == null){
      //      log.warn("Song Tears Don't Fall does not exists!");
            throw new AssertionError();
        }
        Song betrayal = songDao.findById("1");
        if (betrayal == null){
        //    log.warn("Song Your Betrayal does not exists!");
            throw new AssertionError();
        }
        Song letting = songDao.findById("2");
        if (letting == null){
         //   log.warn("Song Letting You Go does not exists!");
            throw new AssertionError();
        }
        Song hearts = songDao.findById("3");
        if (hearts == null){
         //   log.warn("Song Hearts Burst into Fire does not exists!");
            throw new AssertionError();

        }
        Song breathe = songDao.findById("4");
        if (breathe == null){
         //   log.warn("Song Breathe Underwater does not exists!");
            throw new AssertionError();
        }

        Set<Song> songs = new HashSet<>(List.of(tears, betrayal, letting, hearts, breathe));
        bestOf.setSongs(songs);

        Band bftv = bandDao.findById("0");
        if (bftv == null){
         //   log.warn("Band Bullet for My Valentine does not exists!");
            throw new AssertionError();
        }
        bestOf.setBand(bftv);
        albumDao.create(bestOf);
    }

    @Transactional
    public void createManager(){
        Manager managerJohn = new Manager();
        managerJohn.setName("John Doe");
        managerJohn.setId("0");
        Band bfmv  = bandDao.findById("0");
        if (bfmv == null){
        //    log.warn("Band Bullet for My Valentine does not exists!");
            throw new AssertionError();
        }
        managerJohn.addManagedBand(bfmv);
        managerDao.create(managerJohn);

        Manager managerJane = new Manager();
        managerJane.setName("Jane Collins");
        managerJane.setId("1");
        managerDao.create(managerJane);
    }

    @Transactional
    public void createOffers(){
        Offer offer  = new Offer();
        Band bow = bandDao.findById("1");
        if (bow == null){
        //    log.warn("Band Bottle of Watter does not exists!");
            throw new AssertionError();
        }
        offer.setName("Offer to play in the band Bottle of Watter");
        offer.setBand(bow);
        offerDao.create(offer);
    }

    @Transactional
    public void createActivity(){
        Activity activity = new Activity();
        activity.setName("Rock for People");
        Band bfmv  = bandDao.findById("0");
        if (bfmv == null){
         //   log.warn("Band Bullet for My Valentine does not exists!");
            throw new AssertionError();
        }
        activity.setBand(bfmv);
        activity.setId("0");
        activity.setTime(LocalDateTime.of(2022, 6, 15, 0, 0));
        activityDao.create(activity);

        Activity activity1 = new Activity();
        activity1.setTime(LocalDateTime.of(2022, 7,5, 0,0));
        activity1.setName("Concert");
        activity1.setId("1");
        activityDao.create(activity1);
    }
}
