package cz.fi.muni.pa165.entity.security;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import cz.fi.muni.pa165.entity.Musician;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import static java.util.Objects.requireNonNull;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "UserDetails")
public class User implements UserDetails {

    @Id
    @NotNull
    private String id = UUID.randomUUID().toString();

    @NotBlank
    private String name;

    @NotBlank
    private String password;

    @OneToOne
    private Musician musician;

    @NotNull
    private UserRole userRole;

    @JsonCreator
    public User(@JsonProperty("id") final String id,
                      @JsonProperty("name") final String name,
                      @JsonProperty("password") final String password,
                      @JsonProperty("userRole") final String userRole) {
        this.id = requireNonNull(id);
        this.name = requireNonNull(name);
        this.password = requireNonNull(password);
        this.userRole = UserRole.valueOf(requireNonNull(userRole));
    }

    @JsonCreator
    public User(@JsonProperty("id") final String id,
                      @JsonProperty("loginName") final String loginName,
                      @JsonProperty("password") final String password,
                      @JsonProperty("userRole") final String userRole,
                      Musician musician) {
        this(id, loginName, password,userRole);
        this.musician = musician;
    }

    public User(@NotBlank String loginName, @NotBlank String password, @NotNull UserRole userRole) {
        this.name = loginName;
        this.password = password;
        this.userRole = userRole;
    }

    public User(@NotBlank String loginName, @NotBlank String password, @NotNull UserRole userRole, Musician musician) {
        this(loginName, password, userRole);
        this.musician = musician;
    }

    @JsonIgnore
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        List<GrantedAuthority> authorities = new ArrayList<>();
        authorities.add(new SimpleGrantedAuthority(userRole.getRoleName()));
        return authorities;
    }

    @Override
    public String getUsername() {
        return name;
    }

    @JsonIgnore
    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @JsonIgnore
    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @JsonIgnore
    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @JsonIgnore
    @Override
    public boolean isEnabled() {
        return true;
    }

    protected String toStringAdditionalInfo() {
        if (musician != null) {
            return String.format("login'%s', musician:'%s', systemRole: '%s'", getName(), musician.getName(), getUserRole().getRoleName());
        }
        return String.format("login'%s', musician:null, systemRole: '%s'", getName(), getUserRole().getRoleName());
    }
}
