package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.AlbumService;
import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.dto.album.AlbumCreateDto;
import cz.fi.muni.pa165.dto.album.AlbumDto;
import cz.fi.muni.pa165.dto.album.AlbumUpdateDto;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.web.bind.annotation.*;

import javax.inject.Inject;
import javax.validation.Valid;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@RestController
@RequestMapping(ApiUris.ALBUM_ROOT_URI)
public class AlbumController {

    private AlbumService albumService;
    private BandService bandService;

    @Inject
    public AlbumController(AlbumService albumService){
        this.albumService = albumService;
    }

    //find all
    @GetMapping(value = "/", produces = APPLICATION_JSON_VALUE)
    public final List<AlbumDto> getAlbums(){
        return albumService.findAll().stream().map(DtoMapper:: mapToDto).collect(Collectors.toList());
    }
    //create
    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody AlbumCreateDto albumDto) throws ResourceAlreadyExistingException {
        try {
            Band band = bandService.findById(albumDto.getBandId());
            Album album = CreateDtoMapper.mapToEntity(albumDto);
            album.setBand(band);
            albumService.create(album);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    //find by id
    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final AlbumDto getAlbum(@PathVariable("id") long id) throws ResourceNotFoundException {
        return DtoMapper.mapToDto(albumService.findById(String.valueOf(id)));
    }

    //find by name
    //TODO proper testing I am not sure this works
    @GetMapping(value = "/{name}", produces = APPLICATION_JSON_VALUE)
    public final List<AlbumDto> getAlbum(@PathVariable("name") String name) throws ResourceNotFoundException {
        return albumService.findByName(name).stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
    }

    //update
    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody AlbumUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Album album = albumService.findById(dto.getId());
        Album updateAlbum = UpdateDtoMapper.mapToEntity(dto, album);
        updateAlbum.setBand(band);
        albumService.update(updateAlbum);
    }
    //delete
    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Album album = albumService.findById(id);
        albumService.delete(album);
    }

}
