package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.ManagerService;
import cz.fi.muni.pa165.dto.manager.ManagerCreateDto;
import cz.fi.muni.pa165.dto.manager.ManagerDto;
import cz.fi.muni.pa165.dto.manager.ManagerUpdateDto;
import cz.fi.muni.pa165.entity.Manager;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.web.bind.annotation.*;

import javax.inject.Inject;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@RestController
@RequestMapping(ApiUris.MANAGER_ROOT_URI)
public class ManagerController {


    private ManagerService managerService;

    @Inject
    public ManagerController(ManagerService managerService) {
        this.managerService = managerService;
    }

//    @ApiOperation(value = "Find all managers")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = Mu.class),
//    })

    @GetMapping(value = "/", produces = APPLICATION_JSON_VALUE)
    public final List<ManagerDto> getManagers() {
        return managerService
                .findAll()
                .stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toList());
    }

//    @ApiOperation(value = "Find entity by ID")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = CountryDto.class),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final ManagerDto getManager(@PathVariable("id") long id) throws ResourceNotFoundException {
        return DtoMapper.mapToDto(managerService.findById(String.valueOf(id)));
    }

//    @ApiOperation(value = "Creates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody ManagerCreateDto managerCreateDto) throws ResourceAlreadyExistingException {
        try {
            managerService.create(CreateDtoMapper.mapToEntity(managerCreateDto));
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

//    @ApiOperation(value = "Updates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody ManagerUpdateDto dto) {
        Manager manager = managerService.findById(dto.getId());
        Manager updatedManager = UpdateDtoMapper.mapToEntity(dto, manager);
        managerService.update(updatedManager);
    }

//    @ApiOperation(value = "Deletes an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable String id) {
        Manager manager = managerService.findById(id);
        managerService.delete(manager);
    }
}
