package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.MusicianService;
import cz.fi.muni.pa165.dto.musician.MusicianCreateDto;
import cz.fi.muni.pa165.dto.musician.MusicianDto;
import cz.fi.muni.pa165.dto.musician.MusicianUpdateDto;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.web.bind.annotation.*;

import javax.inject.Inject;
import javax.validation.Valid;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@RestController
@RequestMapping(ApiUris.MUSICIAN_ROOT_URI)
public class MusicianController {


    private MusicianService musicianService;
    private BandService bandService;

    @Inject
    public MusicianController(MusicianService musicianService) {
        this.musicianService = musicianService;
    }

//    @ApiOperation(value = "Find all musicians")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = Mu.class),
//    })

    @GetMapping(value = "/", produces = APPLICATION_JSON_VALUE)
    public final List<MusicianDto> getMusicians() {
        var result = musicianService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }

//    @ApiOperation(value = "Find entity by ID")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = CountryDto.class),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final MusicianDto getMusician(@PathVariable("id") long id) throws ResourceNotFoundException {
        MusicianDto musicianDto = DtoMapper.mapToDto(musicianService.findById(String.valueOf(id)));
        return musicianDto;
    }

//    @ApiOperation(value = "Creates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody MusicianCreateDto musicianDto) throws ResourceAlreadyExistingException {
        try {
            Band band = bandService.findById(musicianDto.getBandId());
            Musician musician = CreateDtoMapper.mapToEntity(musicianDto);
            musician.setBand(band);
            musicianService.create(musician);


        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

//    @ApiOperation(value = "Updates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody MusicianUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Musician musician = musicianService.findById(dto.getId());
        Musician updatedMusician = UpdateDtoMapper.mapToEntity(dto, musician);
        updatedMusician.setBand(band);
        musicianService.update(updatedMusician);
    }

//    @ApiOperation(value = "Deletes an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Musician musician = musicianService.findById(id);
        musicianService.delete(musician);
    }

    @GetMapping(value = "/", produces = APPLICATION_JSON_VALUE)
    public final List<MusicianDto> getAvailableMusicians() {
        var result = musicianService.findAvailableMusicians().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }

    public void acceptOffer(){

    }


}
