package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.OfferService;
import cz.fi.muni.pa165.dto.offer.OfferCreateDto;
import cz.fi.muni.pa165.dto.offer.OfferDto;
import cz.fi.muni.pa165.dto.offer.OfferUpdateDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.web.bind.annotation.*;

import javax.inject.Inject;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@RestController
@RequestMapping(ApiUris.OFFER_ROOT_URI)
public class OfferController {
    private OfferService offerService;
    private BandService bandService;

    @Inject
    public OfferController(OfferService offerService) {
        this.offerService = offerService;
    }

//    @ApiOperation(value = "Find all offers")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = Mu.class),
//    })

    @GetMapping(value = "/", produces = APPLICATION_JSON_VALUE)
    public final List<OfferDto> getOffers() {
        var result = offerService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }

    //    @ApiOperation(value = "Find entity by ID")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK", response = CountryDto.class),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final OfferDto getOffer(@PathVariable("id") long id) throws ResourceNotFoundException {
        OfferDto offerDto = DtoMapper.mapToDto(offerService.findById(String.valueOf(id)));
        return offerDto;
    }

    //    @ApiOperation(value = "Creates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody OfferCreateDto offerDto) throws ResourceAlreadyExistingException {
        try {

            Band band = bandService.findById(offerDto.getBandId());
            Offer offer = CreateDtoMapper.mapToEntity(offerDto);
            offer.setBand(band);
            offerService.create(offer);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    //    @ApiOperation(value = "Updates an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody OfferUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Offer offer = offerService.findById(dto.getId());
        Offer updatedOffer = UpdateDtoMapper.mapToEntity(dto, offer);
        updatedOffer.setBand(band);
        offerService.update(updatedOffer);
    }

    //    @ApiOperation(value = "Deletes an entity")
//    @ApiResponses(value = {
//            @ApiResponse(code = 200, message = "OK"),
//            @ApiResponse(code = 404, message = "Entity not found"),
//    })
    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Offer offer = offerService.findById(id);
        offerService.delete(offer);
    }

}
