package cz.fi.muni.pa165.mapper;

import cz.fi.muni.pa165.dto.activity.ActivityDto;
import cz.fi.muni.pa165.dto.album.AlbumDto;
import cz.fi.muni.pa165.dto.band.BandDto;
import cz.fi.muni.pa165.dto.manager.ManagerDto;
import cz.fi.muni.pa165.dto.musician.MusicianDto;
import cz.fi.muni.pa165.dto.offer.OfferDto;
import cz.fi.muni.pa165.dto.song.SongDto;
import cz.fi.muni.pa165.entity.*;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.entity.Song;

import java.util.stream.Collectors;

public class DtoMapper {

    public static MusicianDto mapToDto(Musician musician) {
        MusicianDto musicianDto =  new MusicianDto();
        musicianDto.setName(musician.getName());
        musicianDto.setMusicianType(musician.getType().name());
        musicianDto.setBand(mapToDto(musician.getBand()));
        return musicianDto;
    }

    public static OfferDto mapToDto(Offer offer) {
        OfferDto offerDto =  new OfferDto();
        offerDto.setName(offer.getName());
        offerDto.setBand(mapToDto(offer.getBand()));
        return offerDto;
    }


    public static BandDto mapToDto(Band band) {
        BandDto bandDto =  new BandDto();
        bandDto.setName(band.getName());
        bandDto.setBandType(band.getType().name());
        bandDto.setLogo(band.getLogo());
        bandDto.setActivities(band.getActivities().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        bandDto.setBandMembers(band.getBandMembers().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        bandDto.setBandAlbums(band.getBandAlbums().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return bandDto;
    }

    public static ActivityDto mapToDto(Activity activity) {
        ActivityDto activityDto =  new ActivityDto();
        activityDto.setName(activity.getName());
        activityDto.setBand(mapToDto(activity.getBand()));
        activityDto.setTime(activity.getTime());
        return activityDto;
    }

    public static ManagerDto mapToDto(Manager manager) {
        ManagerDto managerDto =  new ManagerDto();
        managerDto.setName(manager.getName());
        managerDto.setManagedBands(manager.getManagedBands().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return managerDto;
    }

    public static AlbumDto mapToDto(Album album) {
        AlbumDto albumDto =  new AlbumDto();
        albumDto.setName(album.getName());
        albumDto.setBand(mapToDto(album.getBand()));
        albumDto.setSongs(album.getSongs().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return albumDto;
    }

    public static SongDto mapToDto(Song song) {
        SongDto songDto =  new SongDto();
        songDto.setSongName(song.getSongName());
        songDto.setAlbum(mapToDto(song.getAlbum()));
        songDto.setDuration(song.getDuration());
        return songDto;
    }

}
