package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.MusicianDao;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Offer;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class MusicianServiceImpl implements MusicianService{

    private final MusicianDao musicianRepository;

    @Inject
    public MusicianServiceImpl(MusicianDao musicianRepository){
        this.musicianRepository = musicianRepository;
    }

    @Override
    public void create(Musician musician) {
        if (musician == null) {
            throw new IllegalArgumentException("Musician is null");
        }
        musicianRepository.create(musician);
    }

    @Override
    public Musician findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return musicianRepository.findById(id);
    }

    @Override
    public List<Musician> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return musicianRepository.findByName(name);
    }

    @Override
    public Musician update(Musician musician) {
        if (musician == null)
            throw new IllegalArgumentException("Musician is null");

        Musician musicianFromDB = musicianRepository.findById(musician.getId());
        if (musicianFromDB == null)
            throw new DataRetrievalFailureException("Musician not found in database");

        return musicianRepository.update(musician);
    }

    @Override
    public void delete(Musician musician) {
        if (musician == null) {
            throw new IllegalArgumentException("Musician is null");
        }

        Musician musicianFromDB = musicianRepository.findById(musician.getId());
        if (musicianFromDB == null) {
            throw new DataRetrievalFailureException("Musician not found in database");
        }
        musicianRepository.delete(musician);
    }

    @Override
    public List<Musician> findAll() {
        return musicianRepository.findAll();
    }

    @Override
    public void acceptOffer(Offer offer, Musician musician){
        if(offer == null){
            throw new IllegalArgumentException("Offer cannot be null");
        }
        if(musician == null){
            throw new IllegalArgumentException("Musician cannot be null");
        }
        Band band = offer.getBand();
        musician.setBand(band);
        band.addMusician(musician);
    }

    @Override
    public List<Musician> findAvailableMusicians(){
        List<Musician> musicians = musicianRepository.findAvailableMusicians();
        return musicians;
    }
}
