package cz.fi.muni.pa165.security;

import cz.fi.muni.pa165.entity.security.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class PasswordServiceImpl implements PasswordService{
    private final PasswordEncoder passwordEncoder;

    @Autowired
    public PasswordServiceImpl(PasswordEncoder passwordEncoder) {
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public String encodeNewPassword(User user, String oldPlainPassword, String newPlainPassword) {
        if (user == null) throw new IllegalArgumentException("User is null");
        if (oldPlainPassword == null) throw new IllegalArgumentException("Old Plain Password is null");
        if (newPlainPassword == null) throw new IllegalArgumentException("New Plain Password is null");

        if (!passwordEncoder.matches(oldPlainPassword, user.getPassword())) {
            throw new IllegalArgumentException("Old passwords do not match.");
        }

        return passwordEncoder.encode(newPlainPassword);
    }
}
