package cz.fi.muni.pa165.security;

import cz.fi.muni.pa165.dto.SystemUserDto;
import cz.fi.muni.pa165.entity.security.User;

import java.util.List;
import java.util.Optional;

public interface UserService {
    /**
     * Creates a user entity.
     *
     * @param user User entity to create
     * @throws IllegalArgumentException if provided user entity is null
     */
    void create(User user);

    /**
     * Finds user entity for given ID
     *
     * @param id Id of searched user entity
     * @return null if user entity is not found, user entity otherwise
     * @throws IllegalArgumentException if provided ID is null
     */
    User findById(String id);

    /**
     * Finds list of user entities for given name
     *
     * @param name Name of searched user entity
     * @return Empty list if entities are not found, list of User entities otherwise
     */
    List<User> findByName(String name);

    /**
     * Updates a user entity.
     *
     * @param user User entity to update
     * @throws IllegalArgumentException if provided user entity is null, or name is null, or type is null
     */
    User update(User user);

    /**
     * Delete user.
     *
     * @param user User entity to be deleted
     * @throws IllegalArgumentException  if provided user entity is null
     */
    void delete(User user);

    /**
     * Finds all user entity instances.
     *
     * @return all user entities
     */
    List<User> findAll();


    void changePassword(User user, String oldPlainPassword, String newPlainPassword);

    Optional<String> login(String username, String password);


    Optional<SystemUserDto> extractUserFromToken(String token);


    User findByLoginName(String loginName);


    User getAuthenticatedUser();

    void logout(User user);
}
